﻿using System;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows;

namespace TestProject;

/// <summary>
/// 메시지 헬퍼
/// </summary>
public static class MessageHelper
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Import
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 활성 윈도우 설정하기 - SetForegroundWindow(windowHandle)

    /// <summary>
    /// 활성 윈도우 설정하기
    /// </summary>
    /// <param name="windowHandle">윈도우 핸들</param>
    /// <returns>처리 결과</returns>
    [DllImport("user32", SetLastError = true)]
    private static extern bool SetForegroundWindow(IntPtr windowHandle);
    
    #endregion
    #region 키보드 이벤트 발생시키기 - keybd_event(virtualKey, scanCode, flag, extraInformation)

    /// <summary>
    /// 키보드 이벤트 발생시키기
    /// </summary>
    /// <param name="virtualKey">가상 키</param>
    /// <param name="scanCode">스캔 코드</param>
    /// <param name="flag">플래그</param>
    /// <param name="extraInformation">부가 정보</param>
    [DllImport("user32")]
    private static extern void keybd_event(byte virtualKey, byte scanCode, uint flag, uint extraInformation);

    #endregion
    #region 메시지 게시하기 - PostMessage(windowHandle, message, wordParameter, longParameter)

    /// <summary>
    /// 메시지 게시하기
    /// </summary>
    /// <param name="windowHandle">윈도우 핸들</param>
    /// <param name="message">메시지</param>
    /// <param name="wordParameter">WORD 매개 변수</param>
    /// <param name="longParameter">LONG 매개 변수</param>
    /// <returns>처리 결과</returns>
    [DllImport("user32", SetLastError = true)]
    private static extern bool PostMessage(IntPtr windowHandle, uint message, IntPtr wordParameter, IntPtr longParameter);

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// VK_CONTROL
    /// </summary>
    private const byte VK_CONTROL = 0x11;

    /// <summary>
    /// KEYEVENTF_KEYUP
    /// </summary>
    private const int KEYEVENTF_KEYUP = 0x0002;

    /// <summary>
    /// VK_U
    /// </summary>
    private const byte VK_U = 0x55;

    /// <summary>
    /// VK_A
    /// </summary>
    private const byte VK_A = 0x41;

    /// <summary>
    /// VK_C
    /// </summary>
    private const byte VK_C = 0x43;

    /// <summary>
    /// VK_W
    /// </summary>
    private const byte VK_W = 0x57;

    /// <summary>
    /// WM_LBUTTONDOWN
    /// </summary>
    private const uint WM_LBUTTONDOWN = 0x0201;

    /// <summary>
    /// WM_LBUTTONUP
    /// </summary>
    private const uint WM_LBUTTONUP = 0x0202;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Public

    #region 마우스 클릭하기 - ClickMouse(windowHandle, x, y)

    /// <summary>
    /// 마우스 클릭하기
    /// </summary>
    /// <param name="windowHandle">윈도우 핸들</param>
    /// <param name="x">X</param>
    /// <param name="y">Y</param>
    public static void ClickMouse(IntPtr windowHandle, int x, int y)
    {
        PostMessage(windowHandle, WM_LBUTTONDOWN, IntPtr.Zero, MakeLongParameter(x, y));

        Thread.Sleep(50);

        PostMessage(windowHandle, WM_LBUTTONUP, IntPtr.Zero, MakeLongParameter(x, y));
    }

    #endregion
    #region 텍스트 구하기 - GetText(windowHandle)

    /// <summary>
    /// 텍스트 구하기
    /// </summary>
    /// <param name="windowHandle">윈도우 핸들</param>
    /// <returns>텍스트</returns>
    public static string GetText(IntPtr windowHandle)
    {
        if(windowHandle != IntPtr.Zero)
        {
            SetForegroundWindow(windowHandle);

            // CTRL + A 키 메시지를 보낸다.
            SendCTRLKeyCombination(VK_A);

            Thread.Sleep(500);

            // CTRL + C 키 메시지를 보낸다.
            SendCTRLKeyCombination(VK_C);

            Thread.Sleep(100);

            ClickMouse(windowHandle, 1, 150);

            string text = Clipboard.GetText();

            return text;
        }
        else
        {
            return null;
        }
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////// Private

    #region CTRL 키 조합 보내기 - SendCTRLKeyCombination(virtualKey)

    /// <summary>
    /// CTRL 키 조합 보내기
    /// </summary>
    /// <param name="virtualKey">가상 키</param>
    private static void SendCTRLKeyCombination(byte virtualKey)
    {
        keybd_event(VK_CONTROL, 0, 0, 0);

        keybd_event(virtualKey, 0, 0, 0);

        keybd_event(virtualKey, 0, KEYEVENTF_KEYUP, 0);

        keybd_event(VK_CONTROL, 0, KEYEVENTF_KEYUP, 0);
    }

    #endregion
    #region LONG 매개 변수 만들기 - MakeLongParameter(lowValue, highValue)

    /// <summary>
    /// LONG 매개 변수 만들기
    /// </summary>
    /// <param name="lowValue">하위 값</param>
    /// <param name="highValue">상위 값</param>
    /// <returns>LONG 매개 변수</returns>
    private static IntPtr MakeLongParameter(int lowValue, int highValue)
    {
        return (IntPtr)((highValue << 16) | (lowValue & 0xffff));
    }

    #endregion
}