﻿using System.Collections.Generic;
using System.Threading;
using System.Windows;
using System.Windows.Threading;

namespace TestProject;

/// <summary>
/// 메인 윈도우
/// </summary>
public partial class MainWindow : Window
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - MainWindow()

    /// <summary>
    /// 생성자
    /// </summary>
    public MainWindow()
    {
        InitializeComponent();

        this.fetchButton.Click += fetchButton_Click;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Private
    //////////////////////////////////////////////////////////////////////////////// Event

    #region 텍스트 가져오기 버튼 클릭시 처리하기 - fetchButton_Click(sender, e)

    /// <summary>
    /// 텍스트 가져오기 버튼 클릭시 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private void fetchButton_Click(object sender, RoutedEventArgs e)
    {
        this.outputTextBox.Clear();

        Thread thread = new(new ThreadStart(ProcessFetchText));

        thread.SetApartmentState(ApartmentState.STA);

        thread.Start();
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////// Function

    #region 텍스트 가져오기 처리하기- ProcessFetchText()

    /// <summary>
    /// 텍스트 가져오기 처리하기
    /// </summary>
    private void ProcessFetchText()
    {
        List<WindowInformation> windowInformationList = ChromBrowserProcessHelper.GetChromeBrowserWindowInfoList();

        WindowInformation windowInformation = ChromBrowserProcessHelper.GetMainChromeBrowserWindowInformation(windowInformationList);

        if(windowInformation != null)
        {
            string text = ChromBrowserProcessHelper.GetActiveTabText(windowInformation);

            Dispatcher.Invoke
            (
                () =>
                {
                    this.outputTextBox.AppendText(text);

                    WindowHelper.BringToFront(this);
                }
            );
        }
        else
        {
            Dispatcher.Invoke
            (
                () =>
                {
                    this.outputTextBox.AppendText("실행 중인 크롬 브라우저를 찾을 수 없습니다.");

                    WindowHelper.BringToFront(this);
                }
            );
        }
    }

    #endregion
}